//  2021 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

using System;
using System.IO;
using System.Runtime.InteropServices;
using FREngine;

namespace Sample
{
    // Class for loading/unloading FREngine.dll and initializing/deinitializing Engine.
    // Loading is performed in constructor, unloading in Dispose()
    // Throws exceptions when loading fails
    public class EngineLoader : IDisposable
    {
        // Load FineReader Engine with settings stored in SamplesConfig.cs
        public EngineLoader()
        {
            string enginePath = Path.Combine( FreConfig.GetDllFolder(), "FREngine.dll" );
            string customerProjectId = FreConfig.GetCustomerProjectId();
            string licensePath = FreConfig.GetLicensePath();
            string licensePassword = FreConfig.GetLicensePassword();
            string frengineDataFolder = "";
            string tempDataFolder = "";

            Console.WriteLine("Engine path: {0}", enginePath);
            Console.WriteLine("CPID: {0}", customerProjectId);
            Console.WriteLine("License path: {0}", licensePath);
            Console.WriteLine("License password: {0}", licensePassword);

            try
            {
                // Set FREngine.dll path
                FREngine.FREngine.SetFREnginePath(FreConfig.GetDllFolder());
                // Call the InitializeEngine function
                engine = FREngine.FREngine.InitializeEngine(customerProjectId, licensePath, licensePassword, 
                    frengineDataFolder, tempDataFolder, false);
            }
            catch (Exception)
            {
                engine = null;
                // Deleting all objects
                GC.Collect();
                GC.WaitForPendingFinalizers();
                GC.Collect();

                throw;
            }
        }

        // Unload FineReader Engine
        public void Dispose()
        {
            if (engine == null)
            {
                // Engine was not loaded
                return;
            }
            engine = null;
            // Deleting all objects
            GC.Collect();
            GC.WaitForPendingFinalizers();
            GC.Collect();

            FREngine.FREngine.DeinitializeEngine();
        }

        // Returns pointer to FineReader Engine's main object
        public IEngine Engine
        {
            get
            {
                return engine;
            }
        }

        // private variables
        private FREngine.IEngine engine = null;
    }
}
