//  2019 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

// This sample shows how to create user defined dictionary and use it for recognition.

using System;
using System.Windows.Forms;
using System.IO;
using FREngine;

namespace CustomLanguage
{
    public class CustomLanguageForm : System.Windows.Forms.Form
    {
        private Sample.EngineLoader engineLoader = null;
        private System.Windows.Forms.Button goButton;
        private System.Windows.Forms.StatusBar statusBar;
        private System.Windows.Forms.TextBox textBox1;
        private System.Windows.Forms.Button exit;
        private System.Windows.Forms.GroupBox groupBox1;
        private System.Windows.Forms.Label labelAllWords;
        private System.Windows.Forms.Label labelDictinaryWords;

        // name of the file with custom dictionary
        string dictionaryFileName = Path.Combine( FreConfig.GetSamplesFolder(), @"SampleImages\Demo.amd" );

        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;

        public CustomLanguageForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            //
            // TODO: Add any constructor code after InitializeComponent call
            //
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if( components != null )
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.goButton = new System.Windows.Forms.Button();
            this.statusBar = new System.Windows.Forms.StatusBar();
            this.textBox1 = new System.Windows.Forms.TextBox();
            this.exit = new System.Windows.Forms.Button();
            this.groupBox1 = new System.Windows.Forms.GroupBox();
            this.labelDictinaryWords = new System.Windows.Forms.Label();
            this.labelAllWords = new System.Windows.Forms.Label();
            this.groupBox1.SuspendLayout();
            this.SuspendLayout();
            // 
            // goButton
            // 
            this.goButton.Location = new System.Drawing.Point(193, 7);
            this.goButton.Name = "goButton";
            this.goButton.Size = new System.Drawing.Size(88, 28);
            this.goButton.TabIndex = 1;
            this.goButton.Text = "Go";
            this.goButton.Click += new System.EventHandler(this.goButton_Click);
            // 
            // statusBar
            // 
            this.statusBar.Location = new System.Drawing.Point(0, 143);
            this.statusBar.Name = "statusBar";
            this.statusBar.Size = new System.Drawing.Size(286, 22);
            this.statusBar.TabIndex = 8;
            // 
            // textBox1
            // 
            this.textBox1.BorderStyle = System.Windows.Forms.BorderStyle.None;
            this.textBox1.Location = new System.Drawing.Point(8, 8);
            this.textBox1.Multiline = true;
            this.textBox1.Name = "textBox1";
            this.textBox1.ReadOnly = true;
            this.textBox1.Size = new System.Drawing.Size(152, 56);
            this.textBox1.TabIndex = 9;
            this.textBox1.TabStop = false;
            this.textBox1.Text = "Click Go to\r\n 1. Open image file Demo.tif\r\n 2. Read it\r\n 3. Calculate statistics";
            // 
            // exit
            // 
            this.exit.Location = new System.Drawing.Point(193, 42);
            this.exit.Name = "exit";
            this.exit.Size = new System.Drawing.Size(87, 27);
            this.exit.TabIndex = 10;
            this.exit.Text = "Exit";
            this.exit.Click += new System.EventHandler(this.exit_Click);
            // 
            // groupBox1
            // 
            this.groupBox1.Controls.Add(this.labelDictinaryWords);
            this.groupBox1.Controls.Add(this.labelAllWords);
            this.groupBox1.Location = new System.Drawing.Point(8, 72);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(272, 64);
            this.groupBox1.TabIndex = 11;
            this.groupBox1.TabStop = false;
            this.groupBox1.Text = "Results";
            // 
            // labelDictinaryWords
            // 
            this.labelDictinaryWords.Location = new System.Drawing.Point(8, 40);
            this.labelDictinaryWords.Name = "labelDictinaryWords";
            this.labelDictinaryWords.Size = new System.Drawing.Size(232, 16);
            this.labelDictinaryWords.TabIndex = 1;
            this.labelDictinaryWords.Text = "Dictionary words";
            // 
            // labelAllWords
            // 
            this.labelAllWords.Location = new System.Drawing.Point(8, 16);
            this.labelAllWords.Name = "labelAllWords";
            this.labelAllWords.Size = new System.Drawing.Size(232, 23);
            this.labelAllWords.TabIndex = 0;
            this.labelAllWords.Text = "All words ";
            // 
            // CustomLanguageForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(286, 165);
            this.Controls.Add(this.groupBox1);
            this.Controls.Add(this.exit);
            this.Controls.Add(this.textBox1);
            this.Controls.Add(this.statusBar);
            this.Controls.Add(this.goButton);
            this.Name = "CustomLanguageForm";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "CustomLanguage";
            this.groupBox1.ResumeLayout(false);
            this.ResumeLayout(false);

        }
        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main() 
        {    
            Application.Run( new CustomLanguageForm() );
        }

        private void displayMessage( string text )
        {
            statusBar.Text = text;
            this.Update();
        }        

        private void loadEngine()
        {
            if( engineLoader == null ) {
                engineLoader = new Sample.EngineLoader();
            }
        }

        private void loadProfile()
        {
            engineLoader.Engine.LoadPredefinedProfile( "Default" );
        }

        private void unloadEngine()
        {
            if( engineLoader != null ) {
                engineLoader.Dispose();
                engineLoader = null;
            }
        }

        private void setupFREngine()
        {
            displayMessage("Loading profile...");
            loadProfile();

            engineLoader.Engine.ParentWindow = this.Handle.ToInt64();
            engineLoader.Engine.ApplicationTitle = this.Text;
        }

        private void processImage()
        {
            // Create document
            FRDocument document = engineLoader.Engine.CreateFRDocument();

            try {
                // Add image file to document
                displayMessage( "Loading image..." );
                string imagePath = Path.Combine( FreConfig.GetSamplesFolder(), @"SampleImages\Demo.tif" );

                document.AddImageFile( imagePath, null, null );

                //Recognize document
                displayMessage( "Recognizing..." );

                // Create new recognition language
                ITextLanguage textLanguage = makeTextLanguage();

                DocumentProcessingParams documentProcessingParams = engineLoader.Engine.CreateDocumentProcessingParams();
                documentProcessingParams.PageProcessingParams.RecognizerParams.TextLanguage = textLanguage;

                document.Process( documentProcessingParams );

                // Calcualte statistics
                displayMessage( "Calculating statistics..." );
                WordStatisticsCalculator wordStatisticsCalculator = new WordStatisticsCalculator( document );

                // Show statistics
                showStatistics( wordStatisticsCalculator );
            } catch( Exception error ) {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }
            finally {
                // Close document
                document.Close();
            }
        }

        private void processWithEngine()
        {
            try {
                // Setup FREngine
                setupFREngine();

                // Process sample image
                processImage();
            } catch( Exception error ) {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }
        }

        private void goButton_Click( object sender, System.EventArgs e )
        {
            try {
                // Load ABBYY FineReader Engine
                displayMessage( "Initializing Engine..." );
                loadEngine();

                // Process with ABBYY FineReader Engine
                processWithEngine();

                // Unload ABBYY FineReader Engine
                displayMessage( "Deinitializing Engine..." );
                unloadEngine();
            } catch( Exception error ) {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }

            displayMessage( "" );
        }

        private void exit_Click( object sender, System.EventArgs e )
        {
            Close();
        }

        private void makeDictionary()
        {
            // Create new dictionary
            ILanguageDatabase languageDatabase = engineLoader.Engine.CreateLanguageDatabase();
            FREngine.IDictionary dictionary = languageDatabase.CreateNewDictionary( dictionaryFileName, 
                LanguageIdEnum.LI_EnglishUnitedStates );
            dictionary.Name = "SampleUserDictionary";

            // Add words to dictionary
            dictionary.AddWord( "the", 100 );
            dictionary.AddWord( "a", 100 );
            dictionary.AddWord( "an", 100 );
        }

        private ITextLanguage makeTextLanguage()
        {	
            // Create new TextLanguage object
            ILanguageDatabase languageDatabase = engineLoader.Engine.CreateLanguageDatabase();
            ITextLanguage textLanguage = languageDatabase.CreateTextLanguage();

            // Copy all attributes from predefined English language
            ITextLanguage englishLanguage = engineLoader.Engine.PredefinedLanguages.Find( "English" )
                .TextLanguage;
            textLanguage.CopyFrom( englishLanguage );
            textLanguage.InternalName = "SampleTextLanguage";

            // Bind new dictionary to first (and single) BaseLanguage object within TextLanguage
            IBaseLanguage baseLanguage = textLanguage.BaseLanguages[0];

            // Change internal dictionary name to user-defined
            baseLanguage.InternalName = "SampleBaseLanguage";

            //set custom doctionary for base language
            setDictionary( baseLanguage );

            return textLanguage;
        }

        //set custom dictinary for base language
        private void setDictionary(IBaseLanguage baseLanguage)
        {
            //create dictionary file
            makeDictionary();

            // Get collection of dictionary descriptions and remove all items
            IDictionaryDescriptions dictionaryDescriptions = baseLanguage.DictionaryDescriptions;
            dictionaryDescriptions.DeleteAll();

            // Create user dictionary description and add it to the collection
            IDictionaryDescription dictionaryDescription = dictionaryDescriptions.AddNew( DictionaryTypeEnum.DT_UserDictionary );
            UserDictionaryDescription userDictionaryDescription = dictionaryDescription.GetAsUserDictionaryDescription();

            userDictionaryDescription.FileName = dictionaryFileName;
        }

        private void showStatistics( WordStatisticsCalculator wordStatisticsCalculator )
        {
            labelAllWords.Text = "All words count " + wordStatisticsCalculator.AllWordsCount.ToString();
            labelDictinaryWords.Text = "Dictionary words " + wordStatisticsCalculator.DictionaryWordsCount.ToString();
            this.Update();
        }

    }    
}
