//  2019 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

// This class calculates number of dictionary and non dictinary word in the layout


using FREngine;

namespace CustomLanguage
{
    /// <summary>
    /// Class calculate number of dictionary and non dictinary word in the layout
    /// </summary>
    public class WordStatisticsCalculator
    {
        // Count of all words in layout
        int allWordsCount = 0;
        // Count of all dictionary word in layout
        int dictionaryWordsCount = 0;

        public WordStatisticsCalculator( FRDocument document )
        {
            for( int index = 0; index < document.Pages.Count; index++ ){
                calculateStatisticsForLayout( document.Pages[index].Layout );
            }
        }

        public int AllWordsCount {
            get 
            {
                return allWordsCount;
            }
        }

        public int DictionaryWordsCount
        {
            get
            {
                return dictionaryWordsCount;
            }
        }

        private void calculateStatisticsForLayout( ILayout layout )
        {
            ILayoutBlocks blocks = layout.Blocks;
            for( int index = 0; index < blocks.Count; index++ )
            {
                calculateStatisticsForBlock( blocks[index] );
            }
        }

        void calculateStatisticsForBlock( IBlock block )
        {
            if( block.Type == BlockTypeEnum.BT_Text ) 
            {
                calculateStatisticsForText( block.GetAsTextBlock().Text );
            }
        }

        void calculateStatisticsForText( IText text)
        {
            IParagraphs paragraphs = text.Paragraphs;
            for( int index = 0; index < paragraphs.Count; index++ ) 
            {
                calculateStatisticsForParagraph( paragraphs[index] );
            }
        }

        void calculateStatisticsForParagraph( IParagraph paragraph )
        {
            allWordsCount += paragraph.Words.Count;

            for( int index = 0; index < paragraph.Words.Count; index++ ) 
            {
                if( paragraph.Words[index].IsWordFromDictionary ) 
                {
                    dictionaryWordsCount++;
                }
            }
        }
    }
}
