//  2019 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

using System.IO;
using System.Collections.Generic;

namespace Sample
{
    class ImageSource
    {
        public ImageSource( string imagesFolder )
        {
            string[] imageFilePaths = Directory.GetFiles( imagesFolder );
            imageFiles = new List<ImageFile>();
            foreach( var path in imageFilePaths ) {
                imageFiles.Add( new ImageFile( path ) );
            }
            nextIndex = 0;
            processedImageFilesLock = new object();
            processedImageFilesCount = 0;
        }

        public ImageFile GetNextImageFile()
        {
            lock( this ) {
                if( nextIndex < imageFiles.Count ) {
                    return imageFiles[nextIndex++];
                }
            }
            return null;
        }

        public void AddImageFile( ImageFile imageFile )
        {
            lock( this ) {
                imageFiles.Add( imageFile );
            }
        }

        public void IncreaseProcessedImageFilesCount()
        {
            lock( processedImageFilesLock ) {
                processedImageFilesCount++;
            }
        }

        public int GetProcessedImageFilesCount()
        {
            lock( processedImageFilesLock ) {
                return processedImageFilesCount;
            }
        }

        private List<ImageFile> imageFiles;
        private int nextIndex;
        private object processedImageFilesLock;
        private int processedImageFilesCount;
    }

    class ImageFile
    {
        public ImageFile( string filePath )
        {
            FilePath = filePath;
            FailedRunsCount = 0;
        }

        public string FilePath;
        public int FailedRunsCount;
    }
}
