//  2019 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

// This sample demonstrates how to handle events and use callback interfaces 
// of ABBYY FineReader Engine

using System;
using System.Windows.Forms;
using System.IO;

namespace EventsHandling
{
    public class EventsHandlingForm : System.Windows.Forms.Form
    {
        Sample.EngineLoader engineLoader = null;
        private System.Windows.Forms.Button goButton;
        private System.Windows.Forms.StatusBar statusBar;
        private System.Windows.Forms.TextBox textBox1;
        private System.Windows.Forms.ProgressBar progressBar;
        private System.Windows.Forms.Button exitButton;

        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;

        public EventsHandlingForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            //
            // TODO: Add any constructor code after InitializeComponent call
            //
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if ( components != null ) 
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.goButton = new System.Windows.Forms.Button();
            this.statusBar = new System.Windows.Forms.StatusBar();
            this.textBox1 = new System.Windows.Forms.TextBox();
            this.exitButton = new System.Windows.Forms.Button();
            this.progressBar = new System.Windows.Forms.ProgressBar();
            this.SuspendLayout();
            // 
            // goButton
            // 
            this.goButton.Location = new System.Drawing.Point(193, 7);
            this.goButton.Name = "goButton";
            this.goButton.Size = new System.Drawing.Size(88, 28);
            this.goButton.TabIndex = 1;
            this.goButton.Text = "Go";
            this.goButton.Click += new System.EventHandler(this.goButton_Click);
            // 
            // statusBar
            // 
            this.statusBar.Location = new System.Drawing.Point(0, 108);
            this.statusBar.Name = "statusBar";
            this.statusBar.Size = new System.Drawing.Size(286, 22);
            this.statusBar.TabIndex = 8;
            // 
            // textBox1
            // 
            this.textBox1.BorderStyle = System.Windows.Forms.BorderStyle.None;
            this.textBox1.Location = new System.Drawing.Point(8, 8);
            this.textBox1.Multiline = true;
            this.textBox1.Name = "textBox1";
            this.textBox1.ReadOnly = true;
            this.textBox1.Size = new System.Drawing.Size(152, 56);
            this.textBox1.TabIndex = 9;
            this.textBox1.TabStop = false;
            this.textBox1.Text = "Click Go to\r\n 1. Open image file Demo.tif\r\n 2. Read it\r\n 3. Export results to Dem" +
                "o.rtf";
            // 
            // exitButton
            // 
            this.exitButton.Location = new System.Drawing.Point(193, 42);
            this.exitButton.Name = "exitButton";
            this.exitButton.Size = new System.Drawing.Size(87, 27);
            this.exitButton.TabIndex = 10;
            this.exitButton.Text = "Exit";
            this.exitButton.Click += new System.EventHandler(this.exitButton_Click);
            // 
            // progressBar
            // 
            this.progressBar.Location = new System.Drawing.Point(7, 83);
            this.progressBar.Name = "progressBar";
            this.progressBar.Size = new System.Drawing.Size(273, 21);
            this.progressBar.TabIndex = 11;
            // 
            // EventsHandlingForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(286, 130);
            this.Controls.Add(this.progressBar);
            this.Controls.Add(this.exitButton);
            this.Controls.Add(this.textBox1);
            this.Controls.Add(this.statusBar);
            this.Controls.Add(this.goButton);
            this.Name = "EventsHandlingForm";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "EventsHandling";
            this.ResumeLayout(false);

        }
        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main() 
        {    
            Application.Run( new EventsHandlingForm() );
        }

        private void displayMessage( string text )
        {
            statusBar.Text = text;
            this.Update();
        }        

        private void loadEngine()
        {
            if( engineLoader == null ) {
                engineLoader = new Sample.EngineLoader();
            }
        }

        private void loadProfile()
        {
            engineLoader.Engine.LoadPredefinedProfile("Default");
        }

        private void unloadEngine()
        {
            if( engineLoader != null ) {
                engineLoader.Dispose();
                engineLoader = null;
            }
        }

        private void setupFREngine()
        {
            displayMessage("Loading profile...");
            loadProfile();

            engineLoader.Engine.ParentWindow = this.Handle.ToInt64();
            engineLoader.Engine.ApplicationTitle = this.Text;
        }

        private void processImage()
        {
            // Create document
            FREngine.FRDocument document = engineLoader.Engine.CreateFRDocument();

            try {
                // Add image file to document
                displayMessage( "Loading image..." );
                string imagePath = Path.Combine( FreConfig.GetSamplesFolder(), @"SampleImages\Demo.tif" );

                document.AddImageFile( imagePath, null, null );
                
                //Recognize document
                displayMessage( "Recognizing..." );
            
                FREngine.DIFRDocumentEvents_OnProgressEventHandler recognizeOnProgressHandler = 
                    new FREngine.DIFRDocumentEvents_OnProgressEventHandler(
                    document_RecognitionOnProgress );
                document.OnProgress += recognizeOnProgressHandler;

                document.Process( null );

                document.OnProgress -= recognizeOnProgressHandler;

                // Save results
                displayMessage( "Saving results..." );
                FREngine.DIFRDocumentEvents_OnProgressEventHandler exportOnProgressHandler =
                    new FREngine.DIFRDocumentEvents_OnProgressEventHandler(
                    document_ExportOnProgress );
                document.OnProgress += exportOnProgressHandler;

                document.Export( Path.Combine( FreConfig.GetSamplesFolder(), @"SampleImages\Demo.rtf" ),
                    FREngine.FileExportFormatEnum.FEF_RTF, null );

                document.OnProgress -= exportOnProgressHandler;

            } catch( Exception error ) {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }
            finally {
                // Close document
                document.Close();
            }
        }

        private void processWithEngine()
        {
            try {
                // Setup FREngine
                setupFREngine();

                // Process sample image
                processImage();
            } catch( Exception error ) {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }
        }

        private void goButton_Click( object sender, System.EventArgs e )
        {            
            try {
                goButton.Enabled = false;
                // Load ABBYY FineReader Engine
                displayMessage( "Initializing Engine..." );
                loadEngine();

                // Process with ABBYY FineReader Engine
                processWithEngine();

                // Unload ABBYY FineReader Engine
                displayMessage( "Deinitializing Engine..." );
                unloadEngine();
            } catch( Exception error ) {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }

            displayMessage( "" );
            progressBar.Value = 0;
            goButton.Enabled = true;
        }

        private void exitButton_Click( object sender, System.EventArgs e )
        {
            Close();
        }        

        private void document_RecognitionOnProgress(FREngine.FRDocument sender, int Percentage, ref bool cancel)
        {
            displayMessage( "Recognizing... " + Percentage.ToString() + "% recognized" );
            progressBar.Value = Percentage;
            cancel = false;
        }

        private void document_ExportOnProgress(FREngine.FRDocument sender, int Percentage, ref bool cancel)
        {
            displayMessage( "Saving results... " + Percentage.ToString() + "% exported" );
            progressBar.Value = Percentage;
            cancel = false;
        }

    }
}
