//  2019 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

// This sample shows basic steps of ABBYY FineReader Engine usage:
// Initializing, opening image file, recognition and export, deinitializing

using System;
using System.Windows.Forms;
using System.IO;

namespace Hello
{
    public class HelloForm : System.Windows.Forms.Form
    {
        Sample.EngineLoader engineLoader = null;
        private System.Windows.Forms.Button goButton;
        private System.Windows.Forms.StatusBar statusBar;
        private System.Windows.Forms.TextBox textBoxUserHint;
        private System.Windows.Forms.Button exit;

        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;

        public HelloForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            //
            // TODO: Add any constructor code after InitializeComponent call
            //
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if ( components != null ) 
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.goButton = new System.Windows.Forms.Button();
            this.statusBar = new System.Windows.Forms.StatusBar();
            this.textBoxUserHint = new System.Windows.Forms.TextBox();
            this.exit = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // goButton
            // 
            this.goButton.Location = new System.Drawing.Point( 193, 7 );
            this.goButton.Name = "goButton";
            this.goButton.Size = new System.Drawing.Size( 88, 28 );
            this.goButton.TabIndex = 1;
            this.goButton.Text = "Go";
            this.goButton.Click += new System.EventHandler( this.goButton_Click );
            // 
            // statusBar
            // 
            this.statusBar.Location = new System.Drawing.Point( 0, 88 );
            this.statusBar.Name = "statusBar";
            this.statusBar.Size = new System.Drawing.Size( 286, 22 );
            this.statusBar.TabIndex = 8;
            // 
            // textBoxUserHint
            // 
            this.textBoxUserHint.BorderStyle = System.Windows.Forms.BorderStyle.None;
            this.textBoxUserHint.Location = new System.Drawing.Point( 8, 8 );
            this.textBoxUserHint.Multiline = true;
            this.textBoxUserHint.Name = "textBoxUserHint";
            this.textBoxUserHint.ReadOnly = true;
            this.textBoxUserHint.Size = new System.Drawing.Size( 152, 56 );
            this.textBoxUserHint.TabIndex = 9;
            this.textBoxUserHint.TabStop = false;
            this.textBoxUserHint.Text = "Click Go to\r\n 1. Open image file Demo.tif\r\n 2. Read it\r\n 3. Export results to Dem" +
                "o.rtf";
            // 
            // exit
            // 
            this.exit.Location = new System.Drawing.Point( 193, 42 );
            this.exit.Name = "exit";
            this.exit.Size = new System.Drawing.Size( 87, 27 );
            this.exit.TabIndex = 10;
            this.exit.Text = "Exit";
            this.exit.Click += new System.EventHandler( this.exit_Click );
            // 
            // HelloForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size( 5, 13 );
            this.ClientSize = new System.Drawing.Size( 286, 110 );
            this.Controls.Add( this.exit );
            this.Controls.Add( this.textBoxUserHint );
            this.Controls.Add( this.statusBar );
            this.Controls.Add( this.goButton );
            this.Name = "HelloForm";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Hello";
            this.ResumeLayout( false );
            this.PerformLayout();

        }
        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main() 
        {
            Application.Run( new HelloForm() );
        }

        private void displayMessage( string text )
        {
            statusBar.Text = text;
            this.Update();
        }

        private void loadEngine()
        {
            if( engineLoader == null ) {
                engineLoader = new Sample.EngineLoader();
            }
        }

        private void loadProfile()
        {
            engineLoader.Engine.LoadPredefinedProfile("DocumentConversion_Accuracy");
            // Possible profile names are:
            //   "DocumentConversion_Accuracy", "DocumentConversion_Speed",
            //   "DocumentArchiving_Accuracy", "DocumentArchiving_Speed",
            //   "BookArchiving_Accuracy", "BookArchiving_Speed",
            //   "TextExtraction_Accuracy", "TextExtraction_Speed",
            //   "FieldLevelRecognition",
            //   "BarcodeRecognition_Accuracy", "BarcodeRecognition_Speed",
            //   "HighCompressedImageOnlyPdf",
            //   "BusinessCardsProcessing",
            //   "EngineeringDrawingsProcessing",
            //   "Version9Compatibility",
            //   "Default"
        }

        private void unloadEngine()
        {
            if( engineLoader != null ) {
                engineLoader.Dispose();
                engineLoader = null;
            }
        }

        private void setupFREngine()
        {
            displayMessage("Loading predefined profile...");
            loadProfile();

            engineLoader.Engine.ParentWindow = this.Handle.ToInt64();
            engineLoader.Engine.ApplicationTitle = this.Text;
        }

        private void processImage()
        {
            string imagePath = Path.Combine( FreConfig.GetSamplesFolder(), @"SampleImages\Demo.tif" );

            // Don't recognize PDF file with a textual content, just copy it
            if( engineLoader.Engine.IsPdfWithTextualContent( imagePath, null ) ) {
                displayMessage( "Copy results..." );
                string resultPath = Path.Combine( FreConfig.GetSamplesFolder(), 
                    @"SampleImages\Demo_copy.pdf" );
                File.Copy( imagePath, resultPath, true );
                return;
            }

            // Create document
            FREngine.FRDocument document = engineLoader.Engine.CreateFRDocument();

            try {
                // Add image file to document
                displayMessage( "Loading image..." );
                document.AddImageFile( imagePath, null, null );

                // Recognize document
                displayMessage( "Recognizing..." );
                document.Process( null );

                // Save results
                displayMessage( "Saving results..." );

                // Save results to rtf with default parameters
                document.Export( Path.Combine( FreConfig.GetSamplesFolder(), @"SampleImages\Demo.rtf" ),
                    FREngine.FileExportFormatEnum.FEF_RTF, null );

                // Save results to pdf using 'balanced' scenario
                FREngine.PDFExportParams pdfParams = engineLoader.Engine.CreatePDFExportParams();
                pdfParams.Scenario = FREngine.PDFExportScenarioEnum.PES_Balanced;
                document.Export( Path.Combine( FreConfig.GetSamplesFolder(), @"SampleImages\Demo.pdf" ),
                    FREngine.FileExportFormatEnum.FEF_PDF, pdfParams );
            } 
            catch( Exception error ) 
            {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }
            finally {
                // Close document
                document.Close();
            }
        }

        private void processWithEngine()
        {
            try {
                // Setup FREngine
                setupFREngine();

                // Process sample image
                processImage();
            } catch( Exception error ) {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }
        }

        private void goButton_Click( object sender, System.EventArgs e )
        {
            try {
                // Load ABBYY FineReader Engine
                displayMessage( "Initializing Engine..." );
                loadEngine();

                // Process with ABBYY FineReader Engine
                processWithEngine();

                // Unload ABBYY FineReader Engine
                displayMessage( "Deinitializing Engine..." );
                unloadEngine();
            } catch( Exception error ) {
                MessageBox.Show( this, error.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error );
            }

            displayMessage( "" );
        }

        private void exit_Click( object sender, System.EventArgs e )
        {
            Close();
        }
    }
}
